#!/usr/bin/env ruby

##################################################
# EBt ... EseBtron
#  by T.Okada (tokada@da2.so-net.ne.jp)
##################################################

require "qte"
require "qpe"
include Qte
include Qpe

##################################################
# initialilze
##################################################

$version = "0.0.6"

a = QPEApplication.new([$0]+ARGV)
# なぜか utf8 でないとファイルI/Oに失敗する(泣)
#codec = QTextCodec.codecForName('eucJP')
#codec = QTextCodec.codecForName('sjis')
codec = QTextCodec.codecForName('utf8')
a.setDefaultCodec(codec)


##################################################
# 環境ファイル関係のクラス定義
##################################################
class EBtEnv
  # コンストラクタ
  def initialize( envfile )
    @envFile = envfile
  end

  def load
    hash = Hash.new()
    if ( FileTest.file?( @envFile ) ) then
      # ファイルがあるので読み込む
      fp = File.new( @envFile, "r" )
      fp.each do
        |line|
        line = line.chomp
        if ( line =~ /^$/ ) then
          # skip
        else
          a = line.split("=", 2)
          hash[a[0]] = a[1]
        end
      end
      fp.close()
    end
    # 必要なハッシュキーが存在するか確認
    if ( ! hash.has_key?("datapath") ) then
      # hash["datapath"] = "/var/mnt/card/ebt"
      # デフォルトでは本体メモリ or HDD
      if ( FileTest.exist?("/hdd3") ) then
        hash["datapath"] = "/hdd3/ebt"
      else
        hash["datapath"] = "/home/zaurus/ebt"
      end
    end
    if ( ! hash.has_key?("fontname") ) then
      hash["fontname"] = "lcfont"
    end
    if ( ! hash.has_key?("fontsize") ) then
      hash["fontsize"] = "20"
    end
    return hash
  end

  def save( hash )
    fp = File.new( @envFile, "w" )
    hash.each do
      |key, value|
      fp.print( key + "=" + value + "\n" )
    end
    fp.close()
  end
    
end

##################################################
# メインウィンドウのクラス定義
##################################################
class EBtMainWindow < QWidget
  # コンストラクタ
  def initialize( app, path )
    super()
    # キャプションの設定
    setCaption(tr("EBt"))
    @path = path
    # フォントの設定
    setFont( QFont.new( $envHash["fontname"], $envHash["fontsize"].to_i ) )
    # 縦配置のレイアウト
    @layout = QVBoxLayout.new( self )
    # メモ表題入力用
    @lineEditor = QLineEdit.new( self )
    # メモ本文入力用
    @textEditor = QMultiLineEdit.new( self )
#    @textEditor.setWrapColumnOrWidth( QPEApplication.desktop().width - 40 )
    @textEditor.setWrapColumnOrWidth( 400 )
    @textEditor.setWordWrap( 2 )
    @textEditor.setWrapPolicy( 1 )
    # リンク一覧表示用
    @linkList = QListView.new( self )
    @linkList.setMaximumWidth( 200 )
    # ちょこっとテスト
#    @linkList.addColumn(tr("network"))
#    @linkList.addColumn(tr("file"))
    @linkList.addColumn(tr("リンク"))
#    @linkList.addColumn(tr("ファイル"))
#    @linkList.setTreeStepSize( 30 )
#    @linkList.setFont( QFont.new( "Times", 18, QFont::Bold ) )
#    item = QListViewItem.new( @linkList, tr("ルート"), "/root" )
#    QListViewItem.new( item, tr("child1"), "-child1-" )
#    QListViewItem.new( item, tr("child2"), "-child2-" )
#    @linkList.setOpen( item, true )
#    @linkList.setSelected( item, true )
    @linkList.setMultiSelection( false )
    @linkList.setSelectionMode( 0 )

    # 多目的ボタン（２つだけど‥‥）
    @button = QPushButton.new(tr("編集"),self)
    # ステータスバー
    @statusBar = QStatusBar.new( self )
    # ステータスバーの左側のラベル
    @statusLabel = QLabel.new(@statusBar)
    @statusLabel.setText(tr(""))
    @statusBar.addWidget( @statusLabel,1 )
    # ステータスバーの右側のラベル
    @statusLabel2 = QLabel.new(@statusBar)
    @statusLabel2.setText(tr("EBt " + $version ))
    @statusBar.addWidget( @statusLabel2,1,true )
    # メニューバー/ファイルの中身
    @fileMenu = QPopupMenu.new()
    @fileMenu.insertItem(tr("新規(子)"),self,"createChild")
    @fileMenu.insertItem(tr("新規(コピー)"),self,"createCopy")
    @fileMenu.insertSeparator()
    @fileMenu.insertItem(tr("ゴミ探し"),self,"searchTrash")
    @fileMenu.insertItem(tr("ゴミ箱を空にする"),self,"creanTrash")
    @fileMenu.insertSeparator()
    @fileMenu.insertItem(tr("終了"),app,"quit")
    # メニューバー/移動の中身
    @moveMenu = QPopupMenu.new()
    @moveMenu.insertItem(tr("移動"),self,"moveFile")
    @moveMenu.insertItem(tr("移動(ホーム起点)"),self,"moveFileFromHome")
    @moveMenu.insertItem(tr("移動(ルート起点)"),self,"moveFileFromRoot")
    @moveMenu.insertItem(tr("移動(ブックマーク起点)"),self,"moveFileFromBookmark")
    @moveMenu.insertSeparator()
    @moveMenu.insertItem(tr("ホームに移動"),self,"openHome",QAccel::stringToKey(tr("Ctrl+H")))
    @moveMenu.insertItem(tr("ルートに移動"),self,"openRoot",QAccel::stringToKey(tr("Ctrl+R")))
    @moveMenu.insertItem(tr("ごみ箱に移動"),self,"openTrash",QAccel::stringToKey(tr("Ctrl+T")))
    @moveMenu.insertItem(tr("ブックマークに移動"),self,"openBookmark",QAccel::stringToKey(tr("Ctrl+B")))
    # メニューバー/編集の中身
    @editMenu = QPopupMenu.new()
    @editMenu.insertItem(tr("リンク(現在位置から)"),self,"makeLink")
    @editMenu.insertItem(tr("リンク(ホームから)"),self,"makeLinkHome")
    @editMenu.insertItem(tr("リンク(ルートから)"),self,"makeLinkRoot")
    @editMenu.insertItem(tr("リンク(ブックマーク)"),self,"makeLinkBookmark")
    @editMenu.insertSeparator()
    @editMenu.insertItem(tr("検索"),self,"searchText")
    @editMenu.insertSeparator()
    @editMenu.insertItem(tr("このメモをホームにする"),self,"setHomeCurrentMemo")
    @editMenu.insertSeparator()
    @editMenu.insertItem(tr("リンク削除"),self,"deleteLink")
    @editMenu.insertItem(tr("メモ削除"),self,"deleteMemo")
    # メニューバー/メモの中身
    @memoMenu = QPopupMenu.new()
    @memoMenu.insertItem(tr("編集"),self,"memoOpen",QAccel::stringToKey(tr("Ctrl+O")))
    @memoMenu.insertItem(tr("保存"),self,"memoSave",QAccel::stringToKey(tr("Ctrl+S")))
    # メニューバー/オプション/フォントの中身
    @optionFontMenu = QPopupMenu.new()
    @optionFontMenu.insertItem(tr("12"),self,"changeFont12" )
    @optionFontMenu.insertItem(tr("16"),self,"changeFont16" )
    @optionFontMenu.insertItem(tr("18"),self,"changeFont18" )
    @optionFontMenu.insertItem(tr("20"),self,"changeFont20" )
    @optionFontMenu.insertItem(tr("24"),self,"changeFont24" )
    # メニューバー/オプションの中身
    @optionMenu = QPopupMenu.new()
    @optionMenu.insertItem(tr("インプットスタイル用"),self,"changeInputStyle")
    @optionMenu.insertItem(tr("ビュースタイル用"),self,"changeViewStyle")
    @optionMenu.insertSeparator()
    @optionMenu.insertItem(tr("フォント"),@optionFontMenu )
    @optionMenu.insertSeparator()
    @optionMenu.insertItem(tr("ディレクトリ変更"),self,"changeDirectory" )
    @optionMenu.insertSeparator()
    @optionMenu.insertItem(tr("バックアップ"),self,"backupAction" )
    @optionMenu.insertItem(tr("リストア"),self,"restoreAction" )
    # メニューバー本体
    @menuBar = QPEMenuBar.new( self )
    @menuBar.insertItem(tr("ファイル"),@fileMenu)
    @menuBar.insertItem(tr("移動"),@moveMenu)
    @menuBar.insertItem(tr("編集"),@editMenu)
    @menuBar.insertItem(tr("メモ"),@memoMenu)
    @menuBar.insertItem(tr("オプション"),@optionMenu)
    # レイアウトマネージャへの登録
    @layout.addWidget( @menuBar )
    # 横配置レイアウト
    @subLayout1 = QHBoxLayout.new( @layout )
    @subLayout1.addWidget( @linkList, 1 )

    # 更に縦配置レイアウト
    @subLayout2 = QVBoxLayout.new( @subLayout1 )
    @subLayout2.addWidget( @lineEditor )
    @subLayout2.addWidget( @textEditor )
    @subLayout2.addWidget( @button )

#    @layout.addWidget( @lineEditor )
#    @layout.addWidget( @textEditor )
#    @layout.addWidget( @button )
#    @layout.addLayout( @subLayout1 )
    @layout.addWidget( @statusBar )

    # イベントの関連づけ
    Qte::connect( @button, QSIGNAL("clicked()"), self, "buttonAction" )
    # こっちはtree関連
    Qte::connect( @linkList, QSIGNAL("selectionChanged( QListViewItem * )"), self, "treeSelectAction( QListViewItem * )" )
    Qte::connect( @linkList, QSIGNAL("returnPressed( QListViewItem * )"), self, "treeReturnAction( QListViewItem * )" )

    # ビューモードの設定
    setMode(0)
  end

  # バックアップのメイン処理
  def backupAction
    # 編集モードだったら保存して view モードに移行
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode(0)
    dialog = EBtBackupDirDialog.new( self )
    if (dialog.exec() == 1 ) then
      # OK で終わった
      if ( p = dialog.getSelectedPath ) then
        QMessageBox.information( self, tr("情報"), tr("バックアップを開始します。\n「了解」ボタンを押した後、しばらくお待ちください。"), tr("了解") )
        # パスが入っていた
        # @statusLabel.setText(tr("バックアップ中です…"))
        system("cd #{@path} ; tar cf - . | gzip > #{p}")
        # @statusLabel.setText(tr(""))
        QMessageBox.information( self, tr("ふぅ"), tr("バックアップ完了"), tr("了解") )
      end
    end
  end

  # リストアのメイン処理
  def restoreAction
    dialog = EBtBackupDirDialog.new( self )
    if (dialog.exec() == 1 ) then
      # OK で終わった
      if ( p = dialog.getSelectedPath ) then
        # パスが入っていた
        if ( FileTest.exist?( p ) ) then
          r = QMessageBox.warning( self, tr("注意"), tr("リストアすると今のデータは消えます。\nリストアしても良いですか？"), tr("了解"), tr("止める"), nil, 1, 1 )
          if ( r == 0 ) then
            QMessageBox.information( self, tr("情報"), tr("リストアを開始します。\n「了解」ボタンを押した後、しばらくお待ちください。"), tr("了解") )
            # @statusLabel.setText(tr("リストア中です…"))
            system("cd #{@path} ; zcat #{p} | tar xf -")
            # @statusLabel.setText(tr(""))
            QMessageBox.information( self, tr("ふぅ"), tr("リストアを完了しました"), tr("了解") )
            # 先頭に戻る
            openRoot()
            openHome()
          else
            QMessageBox.information( self, tr("ふぅ"), tr("リストア中止"), tr("了解") )
          end
        else
          QMessageBox.information( self, tr("ふぅ"), tr("バックアップファイルがないので\nリストアできません。"), tr("了解") )
        end
      end
    end
  end

  # ディレクトリの変更
  def changeDirectory
    # 編集モードだったら保存して view モードに移行
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode(0)
    dialog = EBtMemoDirDialog.new( self )
    if ( dialog.exec() == 1 ) then
      # OK で終わった
      if ( p = dialog.getSelectedPath ) then
        if ( ! FileTest.directory?(p) ) then
          Dir.mkdir( p )
        end
        # パスを登録する
        @path = p
        # ディレクトリを変更
        openRoot()
        openHome()
        $envHash["datapath"] = p
        $env.save( $envHash )
      end
    end
  end

  # フォントサイズ変更
  def changeFont12
    changeFont( "12" )
  end

  def changeFont16
    changeFont( "16" )
  end

  def changeFont18
    changeFont( "18" )
  end

  def changeFont20
    changeFont( "20" )
  end

  def changeFont24
    changeFont( "24" )
  end

  # フォントサイズを変更
  def changeFont( size )
    $envHash["fontsize"] = size
    # フォントの設定
    setFont( QFont.new( $envHash["fontname"], $envHash["fontsize"].to_i ) )
    $env.save( $envHash )
  end

  # インプットスタイル用のレイアウト
  def changeInputStyle
    @textEditor.setWrapColumnOrWidth( 400 )
  end

  # ビュースタイル用のレイアウト
  def changeViewStyle
    @textEditor.setWrapColumnOrWidth( 270 )
  end

  # メモの修正モードに移行
  def memoOpen
    # view mode から編集モードに移動
    updateTree
    setMode( 1 )
    @lineEditor.setFocus()
  end

  # メモの保存→ビューモードに移行
  def memoSave
    # ファイルの保存
    @file.save( getData )
    updateTree
    # edit mode から view mode に移動
    setMode( 0 )
  end

  # ツリーでEnterキーを押した時の処理
  def treeReturnAction( item )
    fname = String.new( item.text( 1 ).local8Bit )
    # カレントドキュメントを更新
    openFile( fname )
    # ツリーを更新
    updateTree
    # 編集モードに‥‥やめ
    # setMode(1)
    # フォーカス変更
    # @lineEditor.setFocus()
  end

  # ツリーの選択時の処理
  def treeSelectAction( item )
    # 編集モードだったら保存して view モードに移行
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode(0)
    fname = String.new( item.text( 1 ).local8Bit )
    # item に child が無かったら child を登録
    if ( item.childCount == 0 ) then
      fp = EBtFile.new( @path )
      fp.openFile( fname )
      fp.readIndexFile.each do
        |f|
        cfp = EBtFile.new( @path )
        cfp.openFile( f )
        QListViewItem.new( item, tr(cfp.getDoc[0].chomp), f )
      end
    end
    # カレントドキュメントも更新（openFileが使えない..）
    if ( ( fname <=> @file.getBaseFile ) != 0 ) then
      @file = EBtFile.new( @path )
      @file.openFile( fname )
      setData( @file.getDoc() )
    end
  end

  # ツリーの更新
  def updateTree
    @linkList.clear()
    # まず、現在のファイルを登録
    root = QListViewItem.new( @linkList, tr(@file.getDoc[0].chomp), @file.getBaseFile )
    # 次に、ツリーを更新
    @file.readIndexFile.each do
      |f|
      fp = EBtFile.new( @path )
      fp.openFile( f )
      QListViewItem.new( root, tr(fp.getDoc[0].chomp), f )
    end
    @linkList.setOpen( root, true )
    @linkList.setSelected( root, true )
  end

  # モードの設定
  def setMode( mode )
    @mode = mode
    if ( mode == 0 ) then
      # view mode
      @lineEditor.setReadOnly( true )
      @textEditor.setReadOnly( true )
      @lineEditor.setFocusPolicy( 0 )
      @textEditor.setFocusPolicy( 0 )
      @linkList.setFocusPolicy( 3 )
      @button.setText(tr("編集"))
      @linkList.setFocus()
    else
      # edit mode
      @lineEditor.setReadOnly( false )
      @textEditor.setReadOnly( false )
      @lineEditor.setFocusPolicy( 3 )
      @textEditor.setFocusPolicy( 3 )
      @linkList.setFocusPolicy( 0 )
      @button.setText(tr("保存"))
    end
  end

  # ファイルデータを設定する
  def setData( array )
    # まずデータをクリア
    @lineEditor.setText("")
    @textEditor.clear()
    array.each_index do
      |idx|
      if ( idx == 0 ) then
        @lineEditor.setText( tr(array[idx].chomp) )
      else
        @textEditor.insertLine( tr(array[idx].chomp) )
      end
    end
  end

  # ファイルデータを取り込む
  def getData
    array = []
    array[0] = String.new(@lineEditor.text().local8Bit) + "\n"
    count = 1
#    while ( count - 1 ) < @textEditor.numLines()
#      array[count] = String.new(@textEditor.textLine(count-1).local8Bit) + "\n"
#      count += 1
#    end
    array[count] = String.new(@textEditor.text().local8Bit)
    return array
  end

  def openHome
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    index = EBtHomeIndex.new( @path )
    @file = EBtFile.new( @path )
    @file.openFile( index.load )
    setData( @file.getDoc() )
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree更新
    updateTree
  end

  def openRoot
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    @file = EBtFile.new( @path )
    @file.openRoot()
    setData( @file.getDoc() )
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree更新
    updateTree
  end

  def openTrash
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    @file = EBtFile.new( @path )
    @file.openTrash()
    setData( @file.getDoc() )
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree更新
    updateTree
  end

  def openBookmark
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    @file = EBtFile.new( @path )
    @file.openBookmark()
    setData( @file.getDoc() )
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree更新
    updateTree
  end

  # ボタンが押された時のアクション
  def buttonAction
    if ( @mode == 0 ) then
      # view mode から編集モードに移動
      updateTree
      setMode( 1 )
      @lineEditor.setFocus()
    else
      # ファイルの保存
      @file.save( getData )
      updateTree
      # edit mode から view mode に移動
      setMode( 0 )
    end
  end

  # メニュー ファイル/新規(子) のアクション
  def createChild
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    @file = @file.createChild()
    setData( @file.getDoc() )
#    setMode( 1 )
    # tree更新
    updateTree
    # ここでもう一回編集モードに移行
    setMode( 1 )
    # フォーカスの変更
    @lineEditor.setFocus()
  end

  # メニュー ファイル/新規(コピー) のアクション
  def createCopy
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    # index が存在しない場合は skip
    a = @file.readIndexFile
    # まず、相互リンクを張る
    brother = @file.createChild()
    a.each do
      |name|
      link = EBtFile.new( @path )
      link.openFile( name )
      link.makeLink( brother )
    end
    doc = @file.getDoc
    doc[0] = doc[0].chomp + "（コピー）\n"
    @file = brother
    @file.setDoc( doc )
    setData( @file.getDoc() )
#    setMode( 1 )
    # tree更新
    updateTree
    # ここでもう一回編集モードに移行
    setMode( 1 )
    # フォーカスの変更
    @lineEditor.setFocus()
  end

  def openFile( name )
    @file = EBtFile.new( @path )
    @file.openFile( name )
    setData( @file.getDoc() )
    # ツリーの更新処理を追加
    updateTree
  end

  # メニュー ファイル/移動 のアクション
  def moveFile
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode( 1 )
    # ダイアログを表示
    dialog = EBtSelectDialog.new( self )
    dialog.setRearrangeMode( true )
    a = dialog.setCurrentFile( @file )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だから移動不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      dialog.rearrange()
      # カレントのファイルを変更
      openFile( dialog.getSelectFile )
    end
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree いれたお陰でここでもう一回編集モードに移行
    setMode(1)
  end

  # メニュー ファイル/移動(ホーム起点) のアクション
  def moveFileFromHome
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode( 1 )
    # ダイアログを表示
    dialog = EBtSelectDialog.new( self )
    dialog.setRearrangeMode( true )
    home = EBtFile.new( @path )
    home.openHome
    a = dialog.setCurrentFile( home )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だから移動不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      dialog.rearrange()
      # カレントのファイルを変更
      openFile( dialog.getSelectFile )
    end
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree いれたお陰でここでもう一回編集モードに移行
    setMode(1)
  end

  # メニュー ファイル/移動(ルート起点) のアクション
  def moveFileFromRoot
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode( 1 )
    # ダイアログを表示
    dialog = EBtSelectDialog.new( self )
    dialog.setRearrangeMode( true )
    root = EBtFile.new( @path )
    root.openRoot
    a = dialog.setCurrentFile( root )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だから移動不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      dialog.rearrange()
      # カレントのファイルを変更
      openFile( dialog.getSelectFile )
    end
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree いれたお陰でここでもう一回編集モードに移行
    setMode(1)
  end

  # メニュー ファイル/移動(ブックマーク起点) のアクション
  def moveFileFromBookmark
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )
    end
    setMode( 1 )
    # ダイアログを表示
    dialog = EBtSelectDialog.new( self )
    dialog.setRearrangeMode( true )
    root = EBtFile.new( @path )
    root.openBookmark
    a = dialog.setCurrentFile( root )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だから移動不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      dialog.rearrange()
      # カレントのファイルを変更
      openFile( dialog.getSelectFile )
    end
    # フォーカスの変更
    @lineEditor.setFocus()
    # tree いれたお陰でここでもう一回編集モードに移行
    setMode(1)
  end

  # カレントのドキュメントにリンクを設定する(現在位置基準)
  def makeLink
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    setMode( 1 )
    # とりあえずファイル選択ダイアログを開く
    dialog = EBtSelectDialog.new( self )
    a = dialog.setCurrentFile( @file )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だからリンク不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      # カレントのファイルの子として登録する
      cfile = EBtFile.new( @path )
      cfile.openFile( dialog.getSelectFile )
      @file.makeLink( cfile )
    end
    updateTree
  end

  # カレントのドキュメントにリンクを設定する(ホーム基準)
  def makeLinkHome
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    setMode( 1 )
    # とりあえずファイル選択ダイアログを開く
    dialog = EBtSelectDialog.new( self )
    file = EBtFile.new( @path )
    file.openHome
    a = dialog.setCurrentFile( file )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だからリンク不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      # カレントのファイルの子として登録する
      cfile = EBtFile.new( @path )
      cfile.openFile( dialog.getSelectFile )
      @file.makeLink( cfile )
    end
    updateTree
  end

  # カレントのドキュメントにリンクを設定する(ルート基準)
  def makeLinkRoot
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    setMode( 1 )
    # とりあえずファイル選択ダイアログを開く
    dialog = EBtSelectDialog.new( self )
    file = EBtFile.new( @path )
    file.openRoot
    a = dialog.setCurrentFile( file )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だからリンク不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      # カレントのファイルの子として登録する
      cfile = EBtFile.new( @path )
      cfile.openFile( dialog.getSelectFile )
      @file.makeLink( cfile )
    end
    updateTree
  end

  # カレントのドキュメントにリンクを設定する(ブックマークに)
  def makeLinkBookmark
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    setMode( 1 )
    # カレントのファイルの子として登録する
    cfile = EBtFile.new( @path )
    cfile.openBookmark
    @file.makeLink( cfile )
    QMessageBox.information( self, tr("インフォメーション"), tr("ブックマークに登録しました。"), tr("了解") )
    updateTree
  end

  # 未サポートダイアログ
  def unsupportDialog
    QMessageBox.information( self, tr("おっと"), tr("この機能は未実装だっ！"), tr("嫌でも了解") )
  end

  # リンクの削除
  def deleteLink
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    # 移動不可能なダイアログを表示
    dialog = EBtSelectDialog.new( self )
    dialog.setTraceFile( false )
    a = dialog.setCurrentFile( @file )
    if ( a == 0 ) then
      QMessageBox.information( self, tr("おっと"), tr("リンクがありません。"), tr("だから削除不可") )
      return
    elsif ( ( a == 1 ) && ( ( @file.getBaseFile <=> "/rootfile" ) != 0 ) ) then
      QMessageBox.information( self, tr("おっと"), tr("こちらからでは\nリンクを削除できません。"), tr("だから削除不可") )
      return
    end
    if ( dialog.exec() == 1 ) then
      # rootfile と trash 間のリンクは削除できない
      if ( ( ( @file.getBaseFile <=> "/rootfile" ) == 0 ) && ( ( dialog.getSelectFile <=> "/trash" ) == 0 ) ) then
        QMessageBox.information( self, tr("おっと"), tr("root と trash の間の\nリンクは削除できません。"), tr("だから削除不可") )
      elsif ( ( ( @file.getBaseFile <=> "/trash" ) == 0 ) && ( ( dialog.getSelectFile <=> "/rootfile" ) == 0 ) ) then
        QMessageBox.information( self, tr("おっと"), tr("root と trash の間の\nリンクは削除できません。"), tr("だから削除不可") )
      elsif ( ( ( @file.getBaseFile <=> "/rootfile" ) == 0 ) && ( ( dialog.getSelectFile <=> "/bookmark" ) == 0 ) ) then
        QMessageBox.information( self, tr("おっと"), tr("root と bookmark の間の\nリンクは削除できません。"), tr("だから削除不可") )
      elsif ( ( ( @file.getBaseFile <=> "/bookmark" ) == 0 ) && ( ( dialog.getSelectFile <=> "/rootfile" ) == 0 ) ) then
        QMessageBox.information( self, tr("おっと"), tr("root と bookmark の間の\nリンクは削除できません。"), tr("だから削除不可") )
      else
        # リンクを削除
        @file.deleteLink( dialog.getSelectFile )
        QMessageBox.information( self, tr("確認"), tr("リンクを削除しました。"), tr("嫌でも了解する") )
        updateTree
      end
    end
  end

  # メモ削除
  def deleteMemo
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    r = QMessageBox.warning( self, tr("注意"), tr("メモを削除するとメモのリンクがすべて消えます。\n削除しても良いですか？"), tr("了解"), tr("止める"), nil, 1, 1 )
    if ( r == 0 ) then
      # まず、削除可能なメモかどうか調べる。
      if ( ( ( @file.getBaseFile <=> "/bookmark" ) == 0 ) || 
           ( ( @file.getBaseFile <=> "/rootfile" ) == 0 ) ||
           ( ( @file.getBaseFile <=> "/trash" ) == 0 ) ) then
        # 削除不可
        QMessageBox.information( self, tr("おっと"), tr("このメモは削除できません。"), tr("了解") )
      else
        # 削除可能だった
        # まず、ごみ箱にリンクする。
        cfile = EBtFile.new( @path )
        cfile.openTrash
        @file.makeLink( cfile )
        # 次に、ごみ箱以外のリンクを削除する
        @file.readIndexFile.each do
          |f|
          if ( ( f <=> "/trash" ) != 0 ) then
            # ごみ箱以外のリンクを削除する
            @file.deleteLink( f )
          end
        end
        QMessageBox.information( self, tr("確認"), tr("リンクをすべて削除しました。\nメモはごみ箱に移動しました。"), tr("了解") )
        updateTree
      end
    else
      QMessageBox.information( self, tr("確認"), tr("メモの削除を中止しました。"), tr("了解") )
    end
  end

  # ファイルを探してハッシュに登録
  def traceTree( file, hash )
    cfile = EBtFile.new( @path )
    hash[file] = true
    cfile.openFile( file )
    array = cfile.readIndexFile
    array.each do
      |name|
      if ( hash[name] == false ) then
        traceTree( name, hash )
      end
    end
  end

  # ディレクトリ以下を探索。ファイルが hash になければ array に登録
  def traceDirTree( bpath, apath, hash, array )
    Dir.foreach( bpath + apath ) do
      |name|
      napath = apath + "/" + name
      if ( name =~ /^\./ ) then
        # 対象外
        next
      end
      if ( FileTest.directory?( bpath + napath ) ) then
        # ディレクトリを辿る
        traceDirTree( bpath, napath, hash, array )
        next
      end
      if ( napath =~ /\.txt$/ ) then
        # ファイルを確認
        napath[/\.txt$/] = ""
        if ( hash[napath] == false ) then
          array.push( napath )
        end
        next
      end
    end
  end

  # ゴミ探し
  def searchTrash
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    h = Hash.new(false)
    # まずtree全検索
    traceTree( "/rootfile", h )
    # 続いてファイル全検索
    a = []
    traceDirTree( @path, "", h, a )
    # 見つかったゴミをごみ箱にリンク
    file = EBtFile.new( @path )
    file.openTrash
    a.each do
       |name|
       lfile = EBtFile.new( @path )
       lfile.openFile( name )
       lfile.deleteIndexFile
       file.makeLink( lfile )
    end
    QMessageBox.information(self,tr("メッセージ"),tr(sprintf("合計 %d 個のゴミを見つけました。", a.length )),tr("了解"))
  end

  def creanTrash
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    @file = EBtFile.new( @path )
    @file.openTrash
    array = @file.readIndexFile
    array.each do
      |name|
      if ( ( name <=> "/rootfile" ) == 0 ) then
        next
      end
      @file.deleteLink( name )
    end
    setData( @file.getDoc() )
    # ごみ箱を空にしたらツリーを書き直す。
    updateTree
  end

  def searchTextTree( text, result, doc, file, hash )
    cfile = EBtFile.new( @path )
    hash[file] = true
    cfile.openFile( file )
    array = cfile.readIndexFile
    array.each do
      |name|
      if ( hash[name] == false ) then
        searchTextTree( text, result, doc, name, hash )
      end
    end
    # ここで検索
    flag = false
    cfile.getDoc.each do
      |line|
      if ( line.include?( text ) ) then
        flag = true
      end
    end
    if ( flag == true ) then
      result.makeLink( cfile )
      doc.push( cfile.getDoc[0] )
    end
  end

  def searchText
    # 編集モードだったら保存
    if ( @mode == 1 ) then
      @file.save( getData )      
    end
    textDialog = EBtInputDialog.new("検索文字を入力してください", self )
    if ( textDialog.exec() == 0 ) then
      return
    end
    text = textDialog.getText()
    result = EBtFile.new( @path )
    result.setNewFileName
    result.createFile
    doc = []
    doc.push( text + ":検索結果\n" )
    h = Hash.new(false)
    # まずtree全検索
    searchTextTree( text, result, doc, "/rootfile", h )
    # ルート直下とリンク付け
    root = EBtFile.new( @path )
    root.openRoot
    result.makeLink( root )
    result.save( doc )
    # カレントドキュメントを検索結果に変更
    openFile( result.getBaseFile )
    setMode( 0 )
  end

  # 現在表示中のメモを起動時に開くメモとする。
  def setHomeCurrentMemo
    file = EBtHomeIndex.new( @path )
    name = @file.getBaseFile
    file.save( name )
    QMessageBox.information( self, tr("確認"), tr("起動時に自動的に開くメモに設定しました。"), tr("了解") )
  end

end

##################################################
# 文字列入力用ダイアログ
##################################################
class EBtInputDialog < QDialog
  def initialize( caption, parent )
    super( parent, "dialog", true )
    setCaption(tr(caption))
    @inputBox = QLineEdit.new( self )
    @inputBox.resize( 500,32 )
  end

  def getText
    return String.new( @inputBox.text.local8Bit )
  end

end


##################################################
# バックアップ先の選択ダイアログ
##################################################
class EBtBackupDirDialog < QDialog
  def initialize( parent )
    super( parent, "dialog", true )
    setCaption(tr("選択"))
    # 縦配置のレイアウト
    @layout = QVBoxLayout.new( self )
    # 任意のディレクトリにするとめんどくさいので
    # ディレクトリ選択式にしよう
    @buttonGroup = QButtonGroup.new( tr("バックアップ先"), self )
    @layout2 = QVBoxLayout.new( @buttonGroup )
    if ( FileTest.directory?("/hdd3") ) then
      # HDD がある
      @buttonHDD = QRadioButton.new( tr("内蔵HDD"), @buttonGroup )
      @buttonHDD.setChecked( true )
      @layout2.addWidget( @buttonHDD )
    else
      # 内蔵メモリ
      @buttonMEM = QRadioButton.new( tr("本体メモリ"), @buttonGroup )
      @buttonMEM.setChecked( true )
      @layout2.addWidget( @buttonMEM )
    end
    if ( ! FileTest.exist?("/var/mnt/card/NotAvailable") ) then
      # SDカードがある
      @buttonSD = QRadioButton.new( tr("SDメモリカード"), @buttonGroup )
      @buttonSD.setChecked( false )
      @layout2.addWidget( @buttonSD )
    end
    if ( ! FileTest.exist?("/var/mnt/cf/NotAvailable") ) then
      # CFカードがある
      @buttonCF = QRadioButton.new( tr("CFメモリカード"), @buttonGroup )
      @buttonCF.setChecked( false )
      @layout2.addWidget( @buttonCF )
    end
    @layout.addWidget( @buttonGroup )
  end

  # 選択されているディレクトリのパスを返す
  def getSelectedPath
    if ( @buttonHDD != nil && @buttonHDD.isChecked ) then
      return "/hdd3/ebt_backup.tar.gz"
    elsif ( @buttonMEM != nil && @buttonMEM.isChecked ) then
      return "/home/zaurus/ebt_backup.tar.gz"
    elsif ( @buttonSD != nil && @buttonSD.isChecked ) then
      return "/var/mnt/card/ebt_backup.tar.gz"
    elsif ( @buttonCF != nil && @buttonCF.isChecked ) then
      return "/var/mnt/cf/ebt_backup.tar.gz"
    end
    return nil
  end

end


##################################################
# メモ保存用のディレクトリ選択ダイアログ
##################################################
class EBtMemoDirDialog < QDialog
  def initialize( parent )
    super( parent, "dialog", true )
    setCaption(tr("選択"))
    # 縦配置のレイアウト
    @layout = QVBoxLayout.new( self )
    # 任意のディレクトリにするとめんどくさいので
    # ディレクトリ選択式にしよう
    @buttonGroup = QButtonGroup.new( tr("データ記録先"), self )
    @layout2 = QVBoxLayout.new( @buttonGroup )
    if ( FileTest.directory?("/hdd3") ) then
      # HDD がある
      @buttonHDD = QRadioButton.new( tr("内蔵HDD"), @buttonGroup )
      @buttonHDD.setChecked( true )
      @layout2.addWidget( @buttonHDD )
    else
      # 内蔵メモリ
      @buttonMEM = QRadioButton.new( tr("本体メモリ"), @buttonGroup )
      @buttonMEM.setChecked( true )
      @layout2.addWidget( @buttonMEM )
    end
    if ( ! FileTest.exist?("/var/mnt/card/NotAvailable") ) then
      # SDカードがある
      @buttonSD = QRadioButton.new( tr("SDメモリカード"), @buttonGroup )
      @buttonSD.setChecked( false )
      @layout2.addWidget( @buttonSD )
    end
    if ( ! FileTest.exist?("/var/mnt/cf/NotAvailable") ) then
      # CFカードがある
      @buttonCF = QRadioButton.new( tr("CFメモリカード"), @buttonGroup )
      @buttonCF.setChecked( false )
      @layout2.addWidget( @buttonCF )
    end
    @layout.addWidget( @buttonGroup )
  end

  # 選択されているディレクトリのパスを返す
  def getSelectedPath
    if ( @buttonHDD != nil && @buttonHDD.isChecked ) then
      return "/hdd3/ebt"
    elsif ( @buttonMEM != nil && @buttonMEM.isChecked ) then
      return "/home/zaurus/ebt"
    elsif ( @buttonSD != nil && @buttonSD.isChecked ) then
      return "/var/mnt/card/ebt"
    elsif ( @buttonCF != nil && @buttonCF.isChecked ) then
      return "/var/mnt/cf/ebt"
    end
    return nil
  end

end


##################################################
# ファイル選択用のモーダルダイアログ
##################################################
class EBtSelectDialog < QDialog
  def initialize( parent )
    super( parent, "dialog", true )
    # フォントの設定
    setFont( QFont.new( $envHash["fontname"], $envHash["fontsize"].to_i ) )
    # デフォルトモードの設定
    setTraceFile( true )
    setTraceLink( true )
    # window caption の設定
    setCaption(tr("ファイル選択"))

    # メモ表題表示用
    @titleLine = QLineEdit.new( self )
    @titleLine.setGeometry(0,0,600,28)
    @titleLine.setFocusPolicy( 0 )
    @titleLine.setReadOnly( true )
    # QListBox の定義
    @listBox = QListBox.new( self )
    @listBox.setGeometry(0,28,300,300)
    # 中身の表示用（編集不可）
    @textEditor = QMultiLineEdit.new( self )
    @textEditor.setWrapColumnOrWidth( 300 - 40 )
    @textEditor.setWordWrap( 2 )
    @textEditor.setWrapPolicy( 1 )
    @textEditor.setGeometry(300,28,300,150)
    @textEditor.setReadOnly( true )
    @textEditor.setFocusPolicy( 0 )
    # リンクの表示用（編集不可）
    @linkList = QListBox.new( self )
    @linkList.setGeometry(300,178,300,150)
    @linkList.setSelectionMode( 3 )
    @linkList.setFocusPolicy( 0 )

    # イベントの関連づけ
    Qte::connect( @listBox, QSIGNAL("selected(int)"), self, "selectAction(int)" )
    Qte::connect( @listBox, QSIGNAL("highlighted(int)"), self, "hilightAction(int)" )

    # その他動作フラグ
    # 選択しても順序を変えない
    setRearrangeMode( false )
  end

  def setRearrangeMode( mode )
    @rearrange = mode
  end

  def setTraceFile( mode )
    @traceFile = mode
  end

  def setTraceLink( mode )
    @traceLink = mode
  end

  def setData( array )
    @listBox.clear()
    if ( array.length == 0 ) then
      return
    end
    array.each do
      |val|
      @listBox.insertItem(tr(val))
    end
    @listBox.setCurrentItem(0)
    dispDetails(0)
  end

  def selectAction(arg)
    if ( ! @traceFile ) then
      return
    end
    a = @listBox.currentItem()
    file = EBtFile.new( @path )
    file.openFile( @file[a] )
    if ( @rearrange ) then
      @current.updateLink( file )
    end
    setCurrentFile( file )
    #@listBox.changeItem(tr("押されました"), a)
    #@textEditor.insertLine(tr("#{a} 行目が押されました。"))
  end

  def rearrange
    a = @listBox.currentItem()
    file = EBtFile.new( @path )
    file.openFile( @file[a] )
    @current.updateLink( file )
  end

  def hilightAction(arg)
    a = @listBox.currentItem()
    dispDetails(a)
    updateSelectedLink(a)
  end

  # ファイルをオープンする
  def setCurrentFile( cfile )
    @current = cfile
    @path = cfile.getPath()
    @file = []
    @disp = []
    if ( @traceLink ) then
      cfile.readIndexFile.each do
        |f|
        fp = EBtFile.new( @path )
        fp.openFile( f )
        @file.push( f )
        @disp.push( fp.getDoc[0] )
      end
    end
    @titleLine.setText( tr(cfile.getDoc[0]) )
    setData( @disp )
    # 現在選択中のファイルのリンクを表示
    updateSelectedLink(@listBox.currentItem())
    return @disp.length
  end

  def updateSelectedLink(num)
    @linkList.clear()
    cfile = EBtFile.new( @path )
    if ( @file[num] == nil ) then
      return
    end
    cfile.openFile( @file[num] )
    cfile.readIndexFile.each do
      |f|
      fp = EBtFile.new( @path )
      fp.openFile( f )
      @linkList.insertItem( tr(fp.getDoc[0]) )
    end
  end

  # ファイルの詳細の表示
  def dispDetails( num )
    file = EBtFile.new( @path )
    file.openFile( @file[num] )
    array = file.getDoc()
    # 先頭行は捨てる
    array.shift
    @textEditor.clear()
    array.each do
      |l|
      l = l.chomp
      @textEditor.insertLine(tr(l))
    end
  end

  def getCurrentFile
    return @current.getBasefile
  end

  def getSelectFile
    return @file[@listBox.currentItem()]
  end

end


##################################################
# ホームインデックス用の class
##################################################
class EBtHomeIndex
  def initialize( path )
    @path = path
    @name = path + "/home.index"
  end

  def load
    if ( FileTest.exist?( @name ) ) then
      # ファイルが存在するのでそれを読み込む
      fp = File.new( @name, "r" )
      data = fp.readline.chomp
      fp .close()
    else
      # ファイルが存在しないので root を返す
      data = "/rootfile"
    end
    return data
  end

  def save( data )
    fp = File.new( @name, "w" )
    data = fp.puts( data )
    fp .close()
  end

end

##################################################
# ファイルアクセス用の class
##################################################
class EBtFile
  def initialize( path )
    @path = path
  end

  # path を返す
  def getPath
    return( @path )
  end

  # 現在時刻より標準タイトルを返す
  def nowTitle
    time = Time.now
    return time.strftime("%Y/%m/%d %H:%M:%S")
  end

  # ドキュメントの配列を返す
  def getDoc
    return @doc
  end

  # ドキュメントの配列を返す
  def setDoc( doc )
    @doc = doc
  end

  # ファイル create
  def openRoot
    @basefile = "/rootfile"
    @docfile = @path + @basefile + ".txt"
    @indexfile = @path + @basefile + ".index"
    if FileTest.exist?(@docfile) then
      # 存在するのでその中身を読み込み
      fp = File.new(@docfile,"r")
      @doc = fp.readlines()
      fp.close()
    else
      # 存在しない→create
      fp = File.new(@docfile,"w")
      @doc = []
      @doc[0] = "ルートメモ" + "\n"
      @doc[1] = "このメモがすべてのメモの起点になります。\n"
      @doc.each do
        |val|
        fp.print( val )
      end
      fp.close()
    end
    # これで @doc にテキストの中身が設定される
    # 自動的にごみ箱を生成
    trash = EBtFile.new( @path )
    trash.openTrash()
    makeLink( trash )
    # 自動的にブックマークを作る
    bookmark = EBtFile.new( @path )
    bookmark.openBookmark()
    makeLink( bookmark )
  end

  def openTrash
    @basefile = "/trash"
    @docfile = @path + @basefile + ".txt"
    @indexfile = @path + @basefile + ".index"
    if FileTest.exist?(@docfile) then
      # 存在するのでその中身を読み込み
      fp = File.new(@docfile,"r")
      @doc = fp.readlines()
      fp.close()
    else
      # 存在しない→create
      fp = File.new(@docfile,"w")
      @doc = []
      @doc[0] = "ごみ箱" + "\n"
      @doc[1] = "この下のリンクは「ごみ箱を空にする」でリンクが外されます。\n"
      @doc.each do
        |val|
        fp.print( val )
      end
      fp.close()
    end
    # これで @doc にテキストの中身が設定される
  end

  def openBookmark
    @basefile = "/bookmark"
    @docfile = @path + @basefile + ".txt"
    @indexfile = @path + @basefile + ".index"
    if FileTest.exist?(@docfile) then
      # 存在するのでその中身を読み込み
      fp = File.new(@docfile,"r")
      @doc = fp.readlines()
      fp.close()
    else
      # 存在しない→create
      fp = File.new(@docfile,"w")
      @doc = []
      @doc[0] = "ブックマーク" + "\n"
      @doc[1] = "このメモはブックマーク用です。よく使うメモをブックマークに登録しておくと便利に使えるかも知れません。\n"
      @doc.each do
        |val|
        fp.print( val )
      end
      fp.close()
    end
    # これで @doc にテキストの中身が設定される
  end


  # テキストの保存
  def save( array )
    setDoc( array )
    fp = File.new( @docfile, "w" )
    array.each do
      |val|
      fp.print(val)
    end
    fp.close()
  end

  # 新規ファイル名の決定
  def setNewFileName
    # まずディレクトリの作成
    time = Time.now ;
    mpath = time.strftime("/%Y")
    if ( ! FileTest.directory?(@path + mpath) ) then
      Dir.mkdir(@path + mpath)
    end
    mpath = mpath + time.strftime("/%m")
    if ( ! FileTest.directory?(@path + mpath) ) then
      Dir.mkdir(@path + mpath)
    end
    # 次にファイル名の決定
    mfile = mpath + time.strftime("/%d%H%M%S")
    count = 0
    while ( FileTest.exist?( sprintf("%s%s%03d.txt",@path,mfile,count) ) )
      count += 1
    end
    @basefile = sprintf("%s%03d",mfile,count)
    @docfile = sprintf("%s%s%03d.txt",@path,mfile,count)
    @indexfile = sprintf("%s%s%03d.index",@path,mfile,count)
  end

  # ファイルのクリエイト＆デフォルトの内容設定
  def createFile
    # ファイルをオープン
    fp = File.new(@docfile,"w")
    @doc = []
    @doc[0] = nowTitle() + "\n"
    fp.print( @doc[0] )
    fp.close()
  end

  # 既存ファイルのオープン
  def openFile( fname )
    # ファイルをオープン
    @basefile = fname
    @docfile = @path + @basefile + ".txt"
    @indexfile = @path + @basefile + ".index"
    if FileTest.exist?(@docfile) then
      # 存在するのでその中身を読み込み
      fp = File.new(@docfile,"r")
      @doc = fp.readlines()
      fp.close()
    end
  end

  # 既存ファイルのオープン
  def openHome
    # ファイルをオープン
    home = EBtHomeIndex.new( @path )
    openFile( home.load )
  end

  # indexファイルの読み込み
  def readIndexFile
    array = []
    if ( FileTest.exist?( @indexfile ) ) then
      fp = File.new(@indexfile,"r")
      fp.each do
        |line|
        array.push(line.chomp)
      end
      fp.close()
    end
    return array
  end

  # indexファイルの保存
  def saveIndexFile( array )
    fp = File.new( @indexfile, "w" )
    array.each do
      |line|
      fp.print(line+"\n")
    end
    fp.close()
  end

  # ベースファイル名の獲得
  def getBaseFile
    return @basefile
  end

  # リンク順序の変更
  def updateLink( child )
    # 自分自身は登録不可
    if ( ( child.getBaseFile <=> getBaseFile ) == 0 ) then
      # 何もしない
      return
    end
    # まずリンクファイルを読み込む
    link = readIndexFile
    clink = child.readIndexFile
    # 存在するかどうかをチェックする
    if ( link.index(child.getBaseFile) != nil ) then
      # 子が存在する…新たなリンクを作成せず順序を変更
      link.delete(child.getBaseFile)
      link.unshift(child.getBaseFile)
      clink.delete( getBaseFile )
      clink.unshift( getBaseFile )
    else
      # 配列の先頭にそれぞれを設定する
      link.unshift( child.getBaseFile )
      clink.unshift( getBaseFile )
    end
    # リンクファイルの保存
    saveIndexFile( link )
    child.saveIndexFile( clink )
  end

  # 指定されたファイルとの間にリンクを作成
  def makeLink( child )
    # 自分自身は登録不可
    if ( ( child.getBaseFile <=> getBaseFile ) == 0 ) then
      # 何もしない
      return
    end
    # まずリンクファイルを読み込む
    link = readIndexFile
    clink = child.readIndexFile
    # 存在するかどうかをチェックする
    if ( link.index(child.getBaseFile) != nil ) then
      # 子が存在する…何もしない
    else
      # 配列の先頭にそれぞれを設定する
      link.unshift( child.getBaseFile )
      clink.unshift( getBaseFile )
    end
    # リンクファイルの保存
    saveIndexFile( link )
    child.saveIndexFile( clink )
  end

  # リンクされた子のデータを作成
  def createChild()
    # まず、新しいファイルオブジェクトを作成
    child = EBtFile.new( @path )
    # ファイル名の特定
    child.setNewFileName
    # ファイルをcreate
    child.createFile
    # リンクの作成
    makeLink( child )
    # これで終了
    return child
  end

  # ファイルの削除
  def deleteFile
    # rootfile だけは削除しない
    # puts "deleteFile:" + @basefile
    if ( ( @basefile <=> "/rootfile" ) == 0 ) then
      return
    end
    # 削除
    File.delete( @docfile )
    File.delete( @indexfile )
  end

  # リンクの削除
  def deleteLink( name )
    # puts "deleteLink:" + @basefile + " to " + name
    child = EBtFile.new( @path )
    child.openFile( name )
    link = readIndexFile
    clink = child.readIndexFile
    link.delete( child.getBaseFile )
    clink.delete( getBaseFile )
    saveIndexFile( link )
    child.saveIndexFile( clink )
    if ( clink.length == 0 ) then
      # child を削除
      child.deleteFile
    end
  end

  def deleteIndexFile
    if ( FileTest.exist?( @indexfile ) ) then
      File.delete( @indexfile )
    end
  end

end


##################################################
# 汚いが環境設定(一応hashで)
##################################################
$env = EBtEnv.new( "/home/zaurus/Settings/ebt.conf" )
$envHash = Hash.new()
$envHash = $env.load
$env.save( $envHash )
if ( ! FileTest.directory?($envHash["datapath"]) ) then
  # puts "--" + $envHash["datapath"] + "--"
  begin
    Dir.mkdir( $envHash["datapath"] )

  rescue SystemCallError
    QMessageBox.information( nil, tr("？？"), tr("データを読めません"), tr("了解") )
    exit
  end
end

##################################################
# ここからメイン処理
##################################################

widget = EBtMainWindow.new( a, $envHash["datapath"] )
widget.openRoot() # 諸般の都合でこの処理は残す（ブックマークとごみ箱のメンテナンスのため）
widget.openHome()
#a.setMainWidget( widget )
#widget.show()
a.showMainWidget( widget )

a.exec()
